@@ -1,13 +1,13 @@
 /***************************************
-  $Header: /home/amb/wwwoffle/src/RCS/cgi.c 1.10 2003/02/17 17:42:59 amb Exp $
+  $Header: /home/amb/wwwoffle/src/RCS/cgi.c 1.26 2005/10/11 18:34:15 amb Exp $
 
-  WWWOFFLE - World Wide Web Offline Explorer - Version 2.8.
+  WWWOFFLE - World Wide Web Offline Explorer - Version 2.9.
   CGI Execution functions.
   ******************/ /******************
   Written by Paul A. Rombouts
   Modified by Andrew M. Bishop
 
-  This file Copyright 2002,03 Paul A. Rombouts & Andrew M. Bishop
+  This file Copyright 2002,03,04,05 Paul A. Rombouts & Andrew M. Bishop
   It may be distributed under the GNU Public License, version 2, or
   any higher version.  See section COPYING of the GNU Public license
   for conditions under which this file may be redistributed.
@@ -24,12 +24,15 @@
 #include <unistd.h>
 #include <sys/wait.h>
 #include <sys/stat.h>
+#include <fcntl.h>
+#include <errno.h>
 
 #include "wwwoffle.h"
 #include "io.h"
 #include "misc.h"
 #include "errors.h"
 #include "config.h"
+#include "sockets.h"
 #include "version.h"
 
 
@@ -67,7 +70,7 @@
  restriction:
 
   - The Location: header returned from the CGI is not used to send the specified
-    URL to the browser in place of the CGI output but is sent to the browser to
+    URL to the client in place of the CGI output but is sent to the client to
     handle.  This removes the possibility of an infinite recursion in the WWWOFFLE
     server.
 
@@ -75,13 +78,9 @@
 
 /* Local functions */
 
-static int putenv_request(char *file, Header *request_head, /*@null@*/ Body *request_body);
+static int putenv_request(int fd,char *file, Header *request_head, /*@null@*/ Body *request_body);
 static int exec_cgi(int fd, char *file, Header *request_head, /*@null@*/ Body *request_body);
-static void handle_cgi_headers(int fd_in,int fd_out);
-
-
-/* Some global variables used to pass on information about remote client */
-char *client_hostname=NULL, *client_ip=NULL;
+static int handle_cgi_headers(int fd_in,int fd_out);
 
 
 /*++++++++++++++++++++++++++++++++++++++
@@ -129,25 +128,26 @@ void LocalCGI(int fd,URL *Url,char *file,Header *request_head,Body *request_body
     PrintMessage(Warning,"Refuse to run local CGI program file '%s' with root privileges.",file);
 
  if(!exec_success)
-   {
-    lseek(fd,0,SEEK_SET);
-    reinit_io(fd);
-    ftruncate(fd,0);
-
     HTMLMessage(fd,500,"WWWOFFLE Local Program Execution Error",NULL,"ExecCGIFailed",
                 "url",Url->path,
                 NULL);
-   }
 }
 
 
-/* A macro definition that makes environment variable setting a little easier. */
+/*++++++++++++++++++++++++++++++++++++++
+  A macro definition that makes environment variable setting a little easier.
+
+  variable The environment variable that is to be set.
+
+  value The value of the environment variable.
+  ++++++++++++++++++++++++++++++++++++++*/
+
 #define putenv_var_val(variable,value) \
 { \
-  char *envstr = (char *)malloc(sizeof(variable "=")+strlen(value)); \
+  char *envstr=(char *)malloc(sizeof(variable "=")+strlen(value)); \
   strcpy(envstr,variable "="); \
   strcpy(envstr+sizeof(variable),value); \
-  if(putenv(envstr) == -1) \
+  if(putenv(envstr)==-1) \
      return(-1); \
 }
 
@@ -157,6 +157,8 @@ void LocalCGI(int fd,URL *Url,char *file,Header *request_head,Body *request_body
 
   int putenv_request Returns 0 on success or -1 on error.
 
+  int fd The file descriptor for the client.
+
   char *file Filename of script.
 
   Header *request_head The head of the request that was made for this page.
@@ -167,15 +169,17 @@ void LocalCGI(int fd,URL *Url,char *file,Header *request_head,Body *request_body
   the implementation neater by not freeing memory and using same macro for each.
   ++++++++++++++++++++++++++++++++++++++*/
 
-static int putenv_request(char *file, Header *request_head, Body *request_body)
+static int putenv_request(int fd,char *file, Header *request_head, Body *request_body)
 {
  int i;
- char portstr[12];
+ char portstr[MAX_INT_STR+1],*host,*ip;
  URL *Url;
 
+ /*@-mustfreefresh@*/
+
  putenv_var_val("SERVER_SOFTWARE","WWWOFFLE/" WWWOFFLE_VERSION);
 
- putenv_var_val("SERVER_NAME",GetLocalHost(0));
+ putenv_var_val("SERVER_NAME",GetLocalHost());
 
  putenv_var_val("GATEWAY_INTERFACE","CGI/1.1");
 
@@ -191,7 +195,7 @@ static int putenv_request(char *file, Header *request_head, Body *request_body)
  Url=SplitURL(request_head->url);
 
  if(!GetHeader(request_head,"Host"))
-    putenv_var_val("HTTP_HOST",Url->host);
+    putenv_var_val("HTTP_HOST",Url->hostport);
 
  putenv_var_val("PATH_INFO",Url->path);
 
@@ -204,11 +208,18 @@ static int putenv_request(char *file, Header *request_head, Body *request_body)
 
  putenv_var_val("PATH_TRANSLATED",file);
 
- if(client_hostname && client_ip && strcmp(client_ip,client_hostname))
-    putenv_var_val("REMOTE_HOST",client_hostname);
+ if(!SocketRemoteName(fd,&host,&ip,NULL))
+   {
+    if(ip && host && strcmp(ip,host))
+       putenv_var_val("REMOTE_HOST",host);
 
- if(client_ip)
-    putenv_var_val("REMOTE_ADDR",client_ip);
+    if(ip)
+      {
+       char *canonical_ip=CanonicaliseHost(ip);
+       putenv_var_val("REMOTE_ADDR",canonical_ip);
+       free(canonical_ip);
+      }
+   }
 
  for(i=0;i<request_head->n;++i)
    {
@@ -238,30 +249,34 @@ static int putenv_request(char *file, Header *request_head, Body *request_body)
              *p = toupper(*q);
           else if(*q == '-')
              *p = '_';
-          else
+          else                  /* key contains illegal character, ignore key */
             {
-             /* key contains illegal character, ignore key */
              free(envstr);
-             continue;
+             p=NULL;
+             break;
             }
          }
 
-       *p++ = '=';
-       strcpy(p, request_head->val[i]);
+       if(p)
+         {
+          *p++ = '=';
+          strcpy(p, request_head->val[i]);
 
-       if(putenv(envstr) == -1)
-          return(-1);
+          if(putenv(envstr) == -1)
+             return(-1);
+         }
       }
-
    }
 
  if(request_body)
    {
-    char length[12];
+    char length[MAX_INT_STR+1];
     sprintf(length,"%d",request_body->length);
     putenv_var_val("CONTENT_LENGTH",length);
    }
 
+ /*@=mustfreefresh@*/
+
  return(0);
 }
 
@@ -269,7 +284,7 @@ static int putenv_request(char *file, Header *request_head, Body *request_body)
 /*++++++++++++++++++++++++++++++++++++++
   Execute a local file as CGI program.
 
-  int ExecLocalCGI returns 1 on success, 0 on failure.
+  int exec_cgi returns 1 on success, 0 on failure.
 
   int fd The file descriptor to write the output to.
 
@@ -285,117 +300,160 @@ static int exec_cgi(int fd, char *file, Header *request_head, Body *request_body
  int cgi_out[2];
  pid_t childpid;
 
+#define PIPE_READER 0
+#define PIPE_WRITER 1
+
  if(pipe(cgi_out))
    {
-    PrintMessage(Warning, "Cannnot create pipe for local CGI program '%s' output [%!s].", file);
-    return 0;
+    PrintMessage(Warning,"Cannnot create pipe for local CGI program '%s' output [%!s].",file);
+    return(0);
    }
 
  if((childpid = fork()) == -1)
-    PrintMessage(Warning, "Cannnot fork to execute local CGI program '%s' [%!s].", file);
- else if(childpid == 0)   /* The child */
+    PrintMessage(Warning,"Cannnot fork to execute local CGI program '%s' [%!s].",file);
+ else if(childpid==0)   /* The child */
    {
-    if(!strcmp(request_head->method,"POST") || !strcmp(request_head->method,"PUT"))
+    int cgi_in;
+
+    close(cgi_out[PIPE_READER]);
+
+    /* Set up the environment. */
+
+    if(putenv_request(fd,file,request_head,request_body) == -1)
+       PrintMessage(Fatal,"Failed to create environment for local CGI program '%s' [%!s].",file);
+
+    /* Set up stdout (cgi_out[PIPE_WRITER] is the only file descriptor we *must* keep). */
+
+    if(cgi_out[PIPE_WRITER]!=STDOUT_FILENO)
       {
-       int cgi_in=CreateTempSpoolFile();
+       if(dup2(cgi_out[PIPE_WRITER],STDOUT_FILENO)==-1)
+          PrintMessage(Fatal,"Cannnot create standard output for local CGI program '%s' [%!s].",file);
+       close(cgi_out[PIPE_WRITER]);
+      }
 
-       if(cgi_in==-1)
+    /* Create the child process to give us the request body (if present). */
+
+    if(request_body)
+      {
+       int cgi_body[2];
+       pid_t child2pid;
+
+       if(pipe(cgi_body))
          {
-          PrintMessage(Warning, "Cannnot create temporary file for local CGI program '%s' input [%!s].", file);
+          PrintMessage(Fatal,"Cannnot create pipe for local CGI program '%s' input [%!s].",file);
           return(0);
          }
 
-       init_io(cgi_in);
-
-       if(write_data(cgi_in,request_body->content,request_body->length)<0)
+       if((child2pid = fork()) == -1)
+          PrintMessage(Fatal,"Cannnot fork again to create CGI program '%s' data source [%!s].",file);
+       else if(child2pid==0)   /* The child */
          {
-          PrintMessage(Warning, "Failed to write data to local CGI program '%s' [%!s].", file);
-          return(0);
+          if(cgi_body[PIPE_WRITER]!=STDIN_FILENO)         close(STDIN_FILENO);
+          if(cgi_body[PIPE_WRITER]!=STDOUT_FILENO)        close(STDOUT_FILENO);
+          if(cgi_body[PIPE_WRITER]!=STDERR_FILENO)        close(STDERR_FILENO);
+          if(cgi_body[PIPE_WRITER]!=cgi_out[PIPE_WRITER]) close(cgi_out[PIPE_WRITER]);
+
+          close(cgi_body[PIPE_READER]);
+          init_io(cgi_body[PIPE_WRITER]);
+
+          write_data(cgi_body[PIPE_WRITER],request_body->content,request_body->length);
+
+          exit(0);
          }
+       /* else */  /* The parent */
 
-       lseek(cgi_in,0,SEEK_SET);
-       reinit_io(cgi_in);
+       close(cgi_body[PIPE_WRITER]);
 
-       if(cgi_in!=0)
-         {
-          if(dup2(cgi_in,0) == -1)
-             PrintMessage(Fatal,"Cannnot create standard input for local CGI program '%s' [%!s].", file);
-          close(cgi_in);
-         }
-
-       if(putenv_request(file,request_head,request_body) == -1)
-          PrintMessage(Warning, "Failed to create environment for local CGI program '%s' [%!s].", file);
+       cgi_in=cgi_body[PIPE_READER];
       }
     else
-       if(putenv_request(file,request_head,NULL) == -1)
-          PrintMessage(Warning, "Failed to create environment for local CGI program '%s' [%!s].", file);
+       cgi_in=open("/dev/null",O_RDONLY);
 
-    if(cgi_out[1]!=1)
+    /* Set up stdin and stderr. */
+
+    if(cgi_in!=STDIN_FILENO)
       {
-       if(dup2(cgi_out[1],1) == -1)
-          PrintMessage(Fatal,"Cannnot create standard output for local CGI program '%s' [%!s].", file);
-       close(cgi_out[1]);
+       if(dup2(cgi_in,STDIN_FILENO)==-1)
+          PrintMessage(Fatal,"Cannnot create standard input for local CGI program '%s' [%!s].",file);
+       close(cgi_in);
       }
-    close(cgi_out[0]);
 
-    execl(file, file, NULL);
+    if(lseek(STDERR_FILENO,(off_t)0,SEEK_CUR)==-1 && errno==EBADF) /* stderr is not open */
+      {
+       int cgi_err=open("/dev/null",O_WRONLY);
 
-    PrintMessage(Fatal, "Failed to execute local CGI program '%s' [%!s].", file);
+       if(cgi_err!=STDERR_FILENO)
+         {
+          if(dup2(cgi_err,STDERR_FILENO)==-1)
+             PrintMessage(Fatal,"Cannnot create standard error for local CGI program '%s' [%!s].",file);
+          close(cgi_err);
+         }
+      }
+
+    execl(file,file,NULL);
+
+    PrintMessage(Fatal,"Failed to execute local CGI program '%s' [%!s].",file);
    }
  else   /* The parent */
    {
     int statval;
+    int headok;
 
-    PrintMessage(Inform, "Forked local CGI program '%s' (pid=%d).", file,childpid);
+    PrintMessage(Inform,"Forked local CGI program '%s' (pid=%d).",file,childpid);
 
-    close(cgi_out[1]);
-    init_io(cgi_out[0]);
+    close(cgi_out[PIPE_WRITER]);
+    init_io(cgi_out[PIPE_READER]);
 
-    handle_cgi_headers(cgi_out[0],fd);
+    headok=handle_cgi_headers(cgi_out[PIPE_READER],fd);
 
-    finish_io(cgi_out[0]);
-    close(cgi_out[0]);
+    finish_io(cgi_out[PIPE_READER]);
+    close(cgi_out[PIPE_READER]);
 
-    if(waitpid(childpid, &statval, 0) != -1)
+    if(waitpid(childpid,&statval,0)!=-1)
       {
        if(WIFEXITED(statval)) 
          {
-          int exitstatus = WEXITSTATUS(statval);
-          PrintMessage(exitstatus?Warning:Inform,
-                       "Local CGI program '%s' (pid=%d) exited with status %d.", file,childpid,exitstatus);
-          if(exitstatus <=1)
-             return 1;
+          int exitstatus=WEXITSTATUS(statval);
+          PrintMessage(exitstatus?Warning:Inform,"Local CGI program '%s' (pid=%d) exited with status %d.",file,childpid,exitstatus);
+          if(exitstatus<=1)
+             return(1);
          }
        else 
-          PrintMessage(Warning, "Local CGI program '%s' (pid=%d) terminated by signal %d.", file,childpid,WTERMSIG(statval));
+          PrintMessage(Warning,"Local CGI program '%s' (pid=%d) terminated by signal %d.",file,childpid,WTERMSIG(statval));
       }
     else
-       PrintMessage(Warning, "wait failed for local CGI program '%s' (pid=%d) [%!s].", file,childpid);
+       PrintMessage(Warning,"Wait failed for local CGI program '%s' (pid=%d) [%!s].",file,childpid);
+
+    return(headok);
    }
 
- return 0;
+ return(0);
 }
 
 
 /*++++++++++++++++++++++++++++++++++++++
   Handle the CGI headers and create proper headers.
 
+  int handle_cgi_headers Return 1 if we found a good header.
+
   int fd_in The input data from the CGI.
 
   int fd_out The output data to the client.
   ++++++++++++++++++++++++++++++++++++++*/
 
-static void handle_cgi_headers(int fd_in,int fd_out)
+static int handle_cgi_headers(int fd_in,int fd_out)
 {
  Header *cgi_head=NULL;
  char *line=NULL;
  char *val,*head;
- char buffer[READ_BUFFER_SIZE];
+ char buffer[IO_BUFFER_SIZE];
  int n;
 
  line=read_line(fd_in,line);
 
- if(!strncmp(line,"HTTP/",sizeof("HTTP")))
+ if(!line)
+    return(0);
+ else if(!strncmp(line,"HTTP/",(size_t)5))
     cgi_head=CreateHeader(line,0);
  else
    {
@@ -403,9 +461,13 @@ static void handle_cgi_headers(int fd_in,int fd_out)
     AddToHeaderRaw(cgi_head,line);
    }
 
- while((line=read_line(fd_in,line)))
-    if(!AddToHeaderRaw(cgi_head,line))
-       break;
+ if(*line && !isspace(*line))
+    while((line=read_line(fd_in,line)))
+       if(!AddToHeaderRaw(cgi_head,line))
+          break;
+
+ if(line)
+    free(line);
 
  if((val=GetHeader(cgi_head,"Status")))
    {
@@ -445,6 +507,8 @@ static void handle_cgi_headers(int fd_in,int fd_out)
  FreeHeader(cgi_head);
  free(head);
 
- while((n=read_data(fd_in,buffer,READ_BUFFER_SIZE))>0)
+ while((n=read_data(fd_in,buffer,IO_BUFFER_SIZE))>0)
     write_data(fd_out,buffer,n);
+
+ return(1);
 }
