@@ -1,12 +1,12 @@
 /***************************************
-  $Header: /home/amb/wwwoffle/src/RCS/configdata.c 2.143 2003/08/20 18:21:22 amb Exp $
+  $Header: /home/amb/wwwoffle/src/RCS/configdata.c 2.159 2006/02/11 20:00:24 amb Exp $
 
-  WWWOFFLE - World Wide Web Offline Explorer - Version 2.8.
+  WWWOFFLE - World Wide Web Offline Explorer - Version 2.9.
   Configuration data functions.
   ******************/ /******************
   Written by Andrew M. Bishop
 
-  This file Copyright 1997,98,99,2000,01,02,03 Andrew M. Bishop
+  This file Copyright 1997,98,99,2000,01,02,03,04,05,06 Andrew M. Bishop
   It may be distributed under the GNU Public License, version 2, or
   any higher version.  See section COPYING of the GNU Public license
   for conditions under which this file may be redistributed.
@@ -31,9 +31,11 @@
 
 
 #ifndef PATH_MAX
+/*+ The maximum pathname length in characters. +*/
 #define PATH_MAX 4096
 #endif
 
+
 /* StartUp section */
 
 /*+ The IP address to bind for IPv4. +*/
@@ -45,6 +47,9 @@ ConfigItem Bind_IPv6;
 /*+ The port number to use for the HTTP proxy port. +*/
 ConfigItem HTTP_Port;
 
+/*+ The port number to use for the HTTPS proxy port. +*/
+ConfigItem HTTPS_Port;
+
 /*+ The port number to use for the wwwoffle port. +*/
 ConfigItem WWWOFFLE_Port;
 
@@ -64,22 +69,23 @@ ConfigItem UseSyslog;
 ConfigItem PassWord;
 
 /*+ Maximum number of servers  +*/
-ConfigItem MaxServers;          /*+ in total. +*/
-ConfigItem MaxFetchServers;     /*+ for fetching. +*/
+ConfigItem MaxServers,          /*+ in total. +*/
+           MaxFetchServers;     /*+ for fetching. +*/
 
 /*+ The item definitions in the StartUp section. +*/
 static ConfigItemDef startup_itemdefs[]={
  {"bind-ipv4"        ,&Bind_IPv4      ,0,0,Fixed,HostOrNone        ,"0.0.0.0"   },
  {"bind-ipv6"        ,&Bind_IPv6      ,0,0,Fixed,HostOrNone        ,"::"        },
  {"http-port"        ,&HTTP_Port      ,0,0,Fixed,PortNumber        ,NULL        },  /* 2 see InitConfigurationFile() */
- {"wwwoffle-port"    ,&WWWOFFLE_Port  ,0,0,Fixed,PortNumber        ,NULL        },  /* 3 see InitConfigurationFile() */
+ {"https-port"       ,&HTTPS_Port     ,0,0,Fixed,PortNumber        ,NULL        },  /* 3 see InitConfigurationFile() */
+ {"wwwoffle-port"    ,&WWWOFFLE_Port  ,0,0,Fixed,PortNumber        ,NULL        },  /* 4 see InitConfigurationFile() */
  {"spool-dir"        ,&SpoolDir       ,0,0,Fixed,PathName          ,DEF_SPOOLDIR},
  {"run-uid"          ,&WWWOFFLE_Uid   ,0,0,Fixed,UserId            ,"-1"        },
  {"run-gid"          ,&WWWOFFLE_Gid   ,0,0,Fixed,GroupId           ,"-1"        },
  {"use-syslog"       ,&UseSyslog      ,0,0,Fixed,Boolean           ,"yes"       },
  {"password"         ,&PassWord       ,0,0,Fixed,String            ,NULL        },
- {"max-servers"      ,&MaxServers     ,0,0,Fixed,CfgMaxServers     ,NULL        },  /* 9 see InitConfigurationFile() */
- {"max-fetch-servers",&MaxFetchServers,0,0,Fixed,CfgMaxFetchServers,NULL        }   /*10 see InitConfigurationFile() */
+ {"max-servers"      ,&MaxServers     ,0,0,Fixed,CfgMaxServers     ,NULL        },  /*10 see InitConfigurationFile() */
+ {"max-fetch-servers",&MaxFetchServers,0,0,Fixed,CfgMaxFetchServers,NULL        }   /*11 see InitConfigurationFile() */
 };
 
 /*+ The StartUp section. +*/
@@ -105,26 +111,23 @@ ConfigItem ConnectTimeout;
 /*+ The option to retry a failed connection. +*/
 ConfigItem ConnectRetry;
 
-/*+ The list of allowed SSL port numbers. +*/
-ConfigItem SSLAllowPort;
-
 /*+ The permissions for creation of +*/
-ConfigItem DirPerm;             /*+ directories. +*/
-ConfigItem FilePerm;            /*+ files. +*/
+ConfigItem DirPerm,             /*+ directories. +*/
+           FilePerm;            /*+ files. +*/
 
 /*+ The name of a progam to run when changing mode to +*/
-ConfigItem RunOnline;           /*+ online. +*/
-ConfigItem RunOffline;          /*+ offline. +*/
-ConfigItem RunAutodial;         /*+ auto dial. +*/
-ConfigItem RunFetch;            /*+ fetch (start or stop). +*/
+ConfigItem RunOnline,           /*+ online. +*/
+           RunOffline,          /*+ offline. +*/
+           RunAutodial,         /*+ auto dial. +*/
+           RunFetch;            /*+ fetch (start or stop). +*/
 
 /*+ The option to have lock files to stop some problems. +*/
 ConfigItem LockFiles;
 
-/*+ The option to reply to the browser with compressed content encoding. +*/
+/*+ The option to reply to the client with compressed content encoding. +*/
 ConfigItem ReplyCompressedData;
 
-/*+ The option to reply to the browser with chunked transfer encoding. +*/
+/*+ The option to reply to the client with chunked transfer encoding. +*/
 ConfigItem ReplyChunkedData;
 
 /*+ The paths or file extensions that are allowed to be used for CGIs. +*/
@@ -137,17 +140,16 @@ static ConfigItemDef options_itemdefs[]={
  {"dns-timeout"          ,&DNSTimeout         ,0,0,Fixed,TimeSecs   ,"1m"       },
  {"connect-timeout"      ,&ConnectTimeout     ,0,0,Fixed,TimeSecs   ,"30"       },
  {"connect-retry"        ,&ConnectRetry       ,0,0,Fixed,Boolean    ,"no"       },
- {"ssl-allow-port"       ,&SSLAllowPort       ,0,1,Fixed,PortNumber ,NULL       },
- {"dir-perm"             ,&DirPerm            ,0,0,Fixed,FileMode   ,NULL       }, /* 6 see InitConfigurationFile() */
- {"file-perm"            ,&FilePerm           ,0,0,Fixed,FileMode   ,NULL       }, /* 7 see InitConfigurationFile() */
+ {"dir-perm"             ,&DirPerm            ,0,0,Fixed,FileMode   ,NULL       }, /* 5 see InitConfigurationFile() */
+ {"file-perm"            ,&FilePerm           ,0,0,Fixed,FileMode   ,NULL       }, /* 6 see InitConfigurationFile() */
  {"run-online"           ,&RunOnline          ,0,0,Fixed,PathName   ,NULL       },
  {"run-offline"          ,&RunOffline         ,0,0,Fixed,PathName   ,NULL       },
  {"run-autodial"         ,&RunAutodial        ,0,0,Fixed,PathName   ,NULL       },
  {"run-fetch"            ,&RunFetch           ,0,0,Fixed,PathName   ,NULL       },
  {"lock-files"           ,&LockFiles          ,0,0,Fixed,Boolean    ,"no"       },
  {"reply-compressed-data",&ReplyCompressedData,0,0,Fixed,Boolean    ,"no"       },
- {"reply-chunked-data"   ,&ReplyChunkedData   ,0,0,Fixed,Boolean    ,"no"       },
- {"exec-cgi"             ,&ExecCGI            ,0,1,Fixed,Url        ,NULL       }
+ {"reply-chunked-data"   ,&ReplyChunkedData   ,0,0,Fixed,Boolean    ,"yes"      },
+ {"exec-cgi"             ,&ExecCGI            ,0,1,Fixed,UrlWild    ,NULL       }
 };
 
 /*+ Options section. +*/
@@ -164,6 +166,12 @@ ConfigItem PragmaNoCacheOnline;
 /*+ The option to allow or ignore the 'Cache-Control: no-cache' request when online. +*/
 ConfigItem CacheControlNoCacheOnline;
 
+/*+ The option to allow or ignore the 'Cache-Control: max-age=0' request online. +*/
+ConfigItem CacheControlMaxAge0Online;
+
+/*+ The option to force a refresh if the request contains a cookie when online. +*/
+ConfigItem CookiesForceRefreshOnline;
+
 /*+ The maximum age of a cached page to use in preference while online. +*/
 ConfigItem RequestChanged;
 
@@ -192,12 +200,15 @@ ConfigItem TryWithoutPassword;
 ConfigItem IntrDownloadKeep;
 
 /*+ The option to keep on downloading interrupted pages if +*/
-ConfigItem IntrDownloadSize;           /*+ smaller than a given size. +*/
-ConfigItem IntrDownloadPercent;        /*+ more than a given percentage complete. +*/
+ConfigItem IntrDownloadSize,           /*+ smaller than a given size. +*/
+           IntrDownloadPercent;        /*+ more than a given percentage complete. +*/
 
 /*+ The option to keep downloads that time out. +*/
 ConfigItem TimeoutDownloadKeep;
 
+/*+ The option to keep cached pages in case of an error message from the remote server. +*/
+ConfigItem KeepCacheIfNotFound;
+
 /*+ The option to request from the server compressed content encoding. +*/
 ConfigItem RequestCompressedData;
 
@@ -208,6 +219,8 @@ ConfigItem RequestChunkedData;
 static ConfigItemDef onlineoptions_itemdefs[]={
  {"pragma-no-cache"        ,&PragmaNoCacheOnline      ,1,0,Fixed,Boolean   ,"yes"},
  {"cache-control-no-cache" ,&CacheControlNoCacheOnline,1,0,Fixed,Boolean   ,"yes"},
+ {"cache-control-max-age-0",&CacheControlMaxAge0Online,1,0,Fixed,Boolean   ,"yes"},
+ {"cookies-force-refresh"  ,&CookiesForceRefreshOnline,1,0,Fixed,Boolean   ,"no" },
  {"request-changed"        ,&RequestChanged           ,1,0,Fixed,TimeSecs  ,"10m"},
  {"request-changed-once"   ,&RequestChangedOnce       ,1,0,Fixed,Boolean   ,"yes"},
  {"request-expired"        ,&RequestExpired           ,1,0,Fixed,Boolean   ,"no" },
@@ -220,6 +233,7 @@ static ConfigItemDef onlineoptions_itemdefs[]={
  {"intr-download-size"     ,&IntrDownloadSize         ,1,0,Fixed,FileSize  ,"1"  },
  {"intr-download-percent"  ,&IntrDownloadPercent      ,1,0,Fixed,Percentage,"80" },
  {"timeout-download-keep"  ,&TimeoutDownloadKeep      ,1,0,Fixed,Boolean   ,"no" },
+ {"keep-cache-if-not-found",&KeepCacheIfNotFound      ,1,0,Fixed,Boolean   ,"no" },
  {"request-compressed-data",&RequestCompressedData    ,1,0,Fixed,Boolean   ,"yes"},
  {"request-chunked-data"   ,&RequestChunkedData       ,1,0,Fixed,Boolean   ,"yes"}
 };
@@ -238,6 +252,9 @@ ConfigItem PragmaNoCacheOffline;
 /*+ The option to allow or ignore the 'Cache-Control: no-cache' request when offline. +*/
 ConfigItem CacheControlNoCacheOffline;
 
+/*+ The option to allow or ignore the 'Cache-Control: nax-age=0' request offline. +*/
+ConfigItem CacheControlMaxAge0Offline;
+
 /*+ The option to not automatically make requests while offline but to need confirmation. +*/
 ConfigItem ConfirmRequests;
 
@@ -246,10 +263,11 @@ ConfigItem DontRequestOffline;
 
 /*+ The item definitions in the OfflineOptions section. +*/
 static ConfigItemDef offlineoptions_itemdefs[]={
- {"pragma-no-cache"       ,&PragmaNoCacheOffline      ,1,0,Fixed,Boolean,"yes"},
- {"cache-control-no-cache",&CacheControlNoCacheOffline,1,0,Fixed,Boolean,"yes"},
- {"confirm-requests"      ,&ConfirmRequests           ,1,0,Fixed,Boolean,"no" },
- {"dont-request"          ,&DontRequestOffline        ,1,0,Fixed,Boolean,"no" }
+ {"pragma-no-cache"        ,&PragmaNoCacheOffline      ,1,0,Fixed,Boolean,"yes"},
+ {"cache-control-no-cache" ,&CacheControlNoCacheOffline,1,0,Fixed,Boolean,"yes"},
+ {"cache-control-max-age-0",&CacheControlMaxAge0Offline,1,0,Fixed,Boolean,"yes"},
+ {"confirm-requests"       ,&ConfirmRequests           ,1,0,Fixed,Boolean,"no" },
+ {"dont-request"           ,&DontRequestOffline        ,1,0,Fixed,Boolean,"no" }
 };
 
 /*+ OfflineOptions section. +*/
@@ -258,6 +276,38 @@ static ConfigSection offlineoptions_section={"OfflineOptions",
                                              offlineoptions_itemdefs};
 
 
+/* SSLOptions section */
+
+/*+ The option to allow caching of SSL connections. +*/
+ConfigItem SSLEnableCaching;
+
+/*+ The list of allowed SSL hosts and port numbers when tunneling. +*/
+ConfigItem SSLAllowTunnel;
+
+/*+ The list of disallowed SSL hosts and port numbers when tunneling. +*/
+ConfigItem SSLDisallowTunnel;
+
+/*+ The list of allowed SSL hosts and port numbers when caching. +*/
+ConfigItem SSLAllowCache;
+
+/*+ The list of disallowed SSL hosts and port numbers when caching. +*/
+ConfigItem SSLDisallowCache;
+
+/*+ The item definitions in the SSLOptions section. +*/
+static ConfigItemDef ssloptions_itemdefs[]={
+ {"enable-caching" ,&SSLEnableCaching ,0,0,Fixed,Boolean        ,NULL},
+ {"allow-tunnel"   ,&SSLAllowTunnel   ,0,1,Fixed,HostAndPortWild,NULL},
+ {"disallow-tunnel",&SSLDisallowTunnel,0,1,Fixed,HostAndPortWild,NULL},
+ {"allow-cache"    ,&SSLAllowCache    ,0,1,Fixed,HostAndPortWild,NULL},
+ {"disallow-cache" ,&SSLDisallowCache ,0,1,Fixed,HostAndPortWild,NULL}
+};
+
+/*+ The SSLOptions section. +*/
+static ConfigSection ssloptions_section={"SSLOptions",
+                                         sizeof(ssloptions_itemdefs)/sizeof(ConfigItemDef),
+                                         ssloptions_itemdefs};
+
+
 /* FetchOptions section */
 
 /*+ The option to also fetch style sheets. +*/
@@ -350,9 +400,9 @@ ConfigItem EnableHTMLModifications;
 /*+ The option of a tag that can be added to the bottom of the spooled pages with the date and some buttons. +*/
 ConfigItem AddCacheInfo;
 
-/*+ The options to modify the anchor tags in the HTML. +*/
-ConfigItem AnchorModifyBegin[3];
-ConfigItem AnchorModifyEnd[3];
+/*+ The options to modify the anchor tags in the HTML +*/
+ConfigItem AnchorModifyBegin[3], /*+ (before the start tag). +*/
+           AnchorModifyEnd[3];   /*+ (after the end tag). +*/
 
 /*+ The option to disable scripts and scripted actions. +*/
 ConfigItem DisableHTMLScript;
@@ -366,6 +416,9 @@ ConfigItem DisableHTMLStyle;
 /*+ The option to disable the <blink> tag. +*/
 ConfigItem DisableHTMLBlink;
 
+/*+ The option to disable the <marquee> tag. +*/
+ConfigItem DisableHTMLMarquee;
+
 /*+ The option to disable Shockwave Flash animations. +*/
 ConfigItem DisableHTMLFlash;
 
@@ -399,6 +452,9 @@ ConfigItem ReplacementHTMLWebbugImage;
 /*+ The option to demoronise MS characters. +*/
 ConfigItem DemoroniseMSChars;
 
+/*+ The option to fix cyrillic pages written in koi8-r mixed with cp1251. +*/
+ConfigItem FixMixedCyrillic;
+
 /*+ The option to disable animated GIFs. +*/
 ConfigItem DisableAnimatedGIF;
 
@@ -416,6 +472,7 @@ static ConfigItemDef modifyhtml_itemdefs[]={
  {"disable-applet"           ,&DisableHTMLApplet          ,1,0,Fixed,Boolean,"no"},
  {"disable-style"            ,&DisableHTMLStyle           ,1,0,Fixed,Boolean,"no"},
  {"disable-blink"            ,&DisableHTMLBlink           ,1,0,Fixed,Boolean,"no"},
+ {"disable-marquee"          ,&DisableHTMLMarquee         ,1,0,Fixed,Boolean,"no"},
  {"disable-flash"            ,&DisableHTMLFlash           ,1,0,Fixed,Boolean,"no"},
  {"disable-meta-refresh"     ,&DisableHTMLMetaRefresh     ,1,0,Fixed,Boolean,"no"},
  {"disable-meta-refresh-self",&DisableHTMLMetaRefreshSelf ,1,0,Fixed,Boolean,"no"},
@@ -427,6 +484,7 @@ static ConfigItemDef modifyhtml_itemdefs[]={
  {"replace-webbug-images"    ,&ReplaceHTMLWebbugImages    ,1,0,Fixed,Boolean,"no"},
  {"replacement-webbug-image" ,&ReplacementHTMLWebbugImage ,1,0,Fixed,Url    ,"/local/dontget/replacement.gif"},
  {"demoronise-ms-chars"      ,&DemoroniseMSChars          ,1,0,Fixed,Boolean,"no"},
+ {"fix-mixed-cyrillic"       ,&FixMixedCyrillic           ,1,0,Fixed,Boolean,"no"},
  {"disable-animated-gif"     ,&DisableAnimatedGIF         ,1,0,Fixed,Boolean,"no"}
 };
 
@@ -459,7 +517,7 @@ ConfigItem LocalNet;
 
 /*+ The item definitions in the LocalNet section. +*/
 static ConfigItemDef localnet_itemdefs[]={
- {"",&LocalNet,0,1,Host,None,NULL}
+ {"",&LocalNet,0,1,HostWild,None,NULL}
 };
 
 /*+ The LocalNet section. +*/
@@ -475,7 +533,7 @@ ConfigItem AllowedConnectHosts;
 
 /*+ The item definitions in the AllowedConnectHosts section. +*/
 static ConfigItemDef allowedconnecthosts_itemdefs[]={
- {"",&AllowedConnectHosts,0,1,Host,None,NULL}
+ {"",&AllowedConnectHosts,0,1,HostWild,None,NULL}
 };
 
 /*+ The AllowedConnectHosts section. +*/
@@ -569,9 +627,9 @@ static ConfigSection dontcompress_section={"DontCompress",
 /*+ The list of censored headers. +*/
 ConfigItem CensorHeader;
 
-/*+ Flags to cause the 'Referer' header to be mangled. +*/
-ConfigItem RefererSelf;
-ConfigItem RefererSelfDir;
+/*+ Flags to cause the 'Referer' header to be mangled +*/
+ConfigItem RefererSelf,    /*+ to point to itself. +*/
+           RefererSelfDir; /*+ to point to the parent directory. +*/
 
 /*+ A flag to cause a 'User-Agent' header always to be added. +*/
 ConfigItem ForceUserAgent;
@@ -599,8 +657,8 @@ ConfigItem FTPUserName;
 ConfigItem FTPPassWord;
 
 /*+ The information that is needed to allow non-anonymous access, +*/
-ConfigItem FTPAuthUser;         /*+ username +*/
-ConfigItem FTPAuthPass;         /*+ password +*/
+ConfigItem FTPAuthUser,         /*+ username. +*/
+           FTPAuthPass;         /*+ password. +*/
 
 /*+ The item definitions in the FTPOptions section. +*/
 static ConfigItemDef ftpoptions_itemdefs[]={
@@ -642,8 +700,8 @@ static ConfigSection mimetypes_section={"MIMETypes",
 ConfigItem Proxies;
 
 /*+ The information that is needed to allow authorisation headers to be added, +*/
-ConfigItem ProxyAuthUser;       /*+ username +*/
-ConfigItem ProxyAuthPass;       /*+ password +*/
+ConfigItem ProxyAuthUser,       /*+ username. +*/
+           ProxyAuthPass;       /*+ password. +*/
 
 /*+ The SSL proxy to use. +*/
 ConfigItem SSLProxy;
@@ -729,6 +787,7 @@ static ConfigSection *sections[]={&startup_section,
                                   &options_section,
                                   &onlineoptions_section,
                                   &offlineoptions_section,
+                                  &ssloptions_section,
                                   &fetchoptions_section,
                                   &indexoptions_section,
                                   &modifyhtml_section,
@@ -746,7 +805,7 @@ static ConfigSection *sections[]={&startup_section,
                                   &alias_section,
                                   &purge_section};
 
-/*+ The whole file +*/
+/*+ The contents of the whole configuration file. +*/
 ConfigFile CurrentConfig={DEF_CONFDIR "/wwwoffle.conf",sizeof(sections)/sizeof(ConfigSection*),sections};
 
 
@@ -770,7 +829,7 @@ char *ConfigurationFileName(void)
 
 void InitConfigurationFile(char *name)
 {
- static char startup[4][8],options[2][8];
+ static char startup[5][MAX_INT_STR+1],options[2][1+MAX_INT_STR+1];
 
  if(name)
     CurrentConfig.name=name;
@@ -791,12 +850,13 @@ void InitConfigurationFile(char *name)
  /* Default values that cannot be set at compile time. */
 
  sprintf(startup[0],"%d",DEF_HTTP_PORT);         startup_itemdefs[ 2].def_val=startup[0];
- sprintf(startup[1],"%d",DEF_WWWOFFLE_PORT);     startup_itemdefs[ 3].def_val=startup[1];
- sprintf(startup[2],"%d",DEF_MAX_SERVERS);       startup_itemdefs[ 9].def_val=startup[2];
- sprintf(startup[3],"%d",DEF_MAX_FETCH_SERVERS); startup_itemdefs[10].def_val=startup[3];
+ sprintf(startup[1],"%d",DEF_HTTPS_PORT);        startup_itemdefs[ 3].def_val=startup[1];
+ sprintf(startup[2],"%d",DEF_WWWOFFLE_PORT);     startup_itemdefs[ 4].def_val=startup[2];
+ sprintf(startup[3],"%d",DEF_MAX_SERVERS);       startup_itemdefs[10].def_val=startup[3];
+ sprintf(startup[4],"%d",DEF_MAX_FETCH_SERVERS); startup_itemdefs[11].def_val=startup[4];
 
- sprintf(options[0],"0%o",DEF_DIR_PERM);         options_itemdefs[6].def_val=options[0];
- sprintf(options[1],"0%o",DEF_FILE_PERM);        options_itemdefs[7].def_val=options[1];
+ sprintf(options[0],"0%o",DEF_DIR_PERM);         options_itemdefs[5].def_val=options[0];
+ sprintf(options[1],"0%o",DEF_FILE_PERM);        options_itemdefs[6].def_val=options[1];
 
  ftpoptions_itemdefs[1].def_val=DefaultFTPPassWord();
 
@@ -804,7 +864,7 @@ void InitConfigurationFile(char *name)
 
  DefaultConfigFile();
 
- LoggingLevel=ConfigInteger(LogLevel);
+ SyslogLevel=ConfigInteger(LogLevel);
 
  SetDNSTimeout(ConfigInteger(DNSTimeout));
  SetConnectTimeout(ConfigInteger(ConnectTimeout));
@@ -814,14 +874,35 @@ void InitConfigurationFile(char *name)
 /*++++++++++++++++++++++++++++++++++++++
   Read in the configuration file.
 
+  int ReadConfigurationFile Returns 0 on success or 1 on error.
+
   int fd The file descriptor to write the error message to.
   ++++++++++++++++++++++++++++++++++++++*/
 
 int ReadConfigurationFile(int fd)
 {
+ static int first_time=1;
  char *errmsg;
 
- errmsg=ReadConfigFile();
+ if(first_time)
+   {
+    errmsg=ReadConfigFile(1);
+
+    if(errmsg)
+      {
+       if(fd!=-1)
+          write_string(fd,errmsg);
+       free(errmsg);
+
+       return(1);
+      }
+
+    SetLocalPort(ConfigInteger(HTTP_Port));
+
+    first_time=0;
+   }
+
+ errmsg=ReadConfigFile(0);
 
  if(errmsg)
    {
@@ -831,15 +912,13 @@ int ReadConfigurationFile(int fd)
 
     return(1);
    }
- else
-   {
-    LoggingLevel=ConfigInteger(LogLevel);
 
-    SetDNSTimeout(ConfigInteger(DNSTimeout));
-    SetConnectTimeout(ConfigInteger(ConnectTimeout));
+ SyslogLevel=ConfigInteger(LogLevel);
 
-    return(0);
-   }
+ SetDNSTimeout(ConfigInteger(DNSTimeout));
+ SetConnectTimeout(ConfigInteger(ConnectTimeout));
+
+ return(0);
 }
 
 
